@@ -3,7 +3,7 @@
 
 import tests
 
-from tests import convert_path_sep
+from tests import os_native_path
 
 import os
 
@@ -12,6 +12,8 @@ from gi.repository import Gdk
 
 import zim.formats
 
+from zim.newfs import LocalFile
+
 from zim.gui.clipboard import *
 
 
@@ -141,26 +143,26 @@ some <b>bold</b> text
 		self.assertTrue(Clipboard.get_parsetree() is None)
 
 	def testCopyTextPasteAsParseTree(self):
-		wanted = '''<?xml version='1.0' encoding='utf-8'?>\n<zim-tree partial="True">some **string**</zim-tree>'''
+		wanted = '''<?xml version='1.0' encoding='utf-8'?>\n<zim-tree>some **string**</zim-tree>'''
 		Clipboard.set_text('some **string**')
 		newtree = Clipboard.get_parsetree(self.notebook)
 		self.assertEqual(newtree.tostring(), wanted)
 
 	def testCopyTextPasteAsParseTreeWiki(self):
-		wanted = '''<?xml version='1.0' encoding='utf-8'?>\n<zim-tree partial="True"><p>some <strong>string</strong></p></zim-tree>'''
+		wanted = '''<?xml version='1.0' encoding='utf-8'?>\n<zim-tree><p>some <strong>string</strong></p></zim-tree>'''
 		Clipboard.set_text('some **string**')
 		newtree = Clipboard.get_parsetree(self.notebook, text_format='wiki')
 		self.assertEqual(newtree.tostring(), wanted)
 
 	def testCopyTextPasteAsParseTreeVerbatim(self):
-		wanted = '''<?xml version='1.0' encoding='utf-8'?>\n<zim-tree partial="True"><pre>some **string**</pre></zim-tree>'''
+		wanted = '''<?xml version='1.0' encoding='utf-8'?>\n<zim-tree><pre>some **string**</pre></zim-tree>'''
 		Clipboard.set_text('some **string**')
 		newtree = Clipboard.get_parsetree(self.notebook, text_format='verbatim-pre')
 		self.assertEqual(newtree.tostring(), wanted)
 
 	def testCopyURLasTextPasteAsParseTree(self):
 		# Special case, e.g. copy URL from browser address bar as text
-		wanted = '''<?xml version='1.0' encoding='utf-8'?>\n<zim-tree partial="True"><link href="https://localhost">https://localhost</link></zim-tree>'''
+		wanted = '''<?xml version='1.0' encoding='utf-8'?>\n<zim-tree><link href="https://localhost">https://localhost</link></zim-tree>'''
 		Clipboard.set_text('https://localhost')
 		newtree = Clipboard.get_parsetree(self.notebook)
 		self.assertEqual(newtree.tostring(), wanted)
@@ -174,11 +176,11 @@ some <b>bold</b> text
 
 	def _testCopyFileURIPasteAsParseTree(self, set_func):
 		page = self.notebook.get_page(Path('Test:wiki'))
-		file = File('/foo/bar/baz.txt')
+		file = LocalFile(os_native_path('/foo/bar/baz.txt'))
 		set_func(file.uri)
 		tree = Clipboard.get_parsetree(self.notebook, page)
-		link = tree.find('link')
-		rel_path = link.get('href')
+		link = tree.find_element('link')
+		rel_path = link.attrib.get('href')
 		self.assertEqual(self.notebook.resolve_file(rel_path, page), file)
 
 	@tests.expectedFailure
@@ -190,13 +192,11 @@ some <b>bold</b> text
 
 	def _testCopyImageFileURIPasteAsParseTree(self, set_func):
 		page = self.notebook.get_page(Path('Test:wiki'))
-		file = File('./data/zim.png') # image file
+		file = tests.ZIM_DATA_FOLDER.file('./zim.png') # image file
 		set_func(file.uri)
 		tree = Clipboard.get_parsetree(self.notebook, page)
-		img = tree.find('img')
-		file_obj = img.get('_src_file')
-		self.assertEqual(file_obj, file)
-		rel_path = img.get('src')
+		img = tree.find_element('img')
+		rel_path = img.attrib.get('src')
 		self.assertEqual(self.notebook.resolve_file(rel_path, page), file)
 
 	@tests.expectedFailure
@@ -210,8 +210,7 @@ some <b>bold</b> text
 		set_func('http://cpan.org', 'ftp://foo@test.org', 'user@mail.com')
 		page = self.notebook.get_page(Path('Test:wiki'))
 		tree = Clipboard.get_parsetree(self.notebook, page)
-		links = tree.findall('link')
-		hrefs = [e.attrib['href'] for e in links]
+		hrefs = [e.attrib['href'] for e in tree.iter_elements('link')]
 		self.assertEqual(hrefs,
 			['http://cpan.org', 'ftp://foo@test.org', 'user@mail.com'])
 
@@ -221,17 +220,12 @@ some <b>bold</b> text
 		self.notebook.get_attachments_dir = lambda p: LocalFolder(inner(p).path) # fixture to ensure local folder used
 
 		page = self.notebook.get_page(Path('Test:wiki'))
-		file = File('./data/zim.png')
+		file = tests.ZIM_DATA_FOLDER.file('./data/zim.png')
 		set_clipboard_image(file)
 		tree = Clipboard.get_parsetree(self.notebook, page)
-		img = tree.find('img')
-		file_obj = img.get('_src_file')
-		self.assertFalse(file_obj == file)
-		self.assertTrue(file_obj.exists())
-		self.assertTrue(file_obj.isimage())
-		self.assertTrue(file_obj.path.endswith('.png'))
-		rel_path = img.get('src')
-		self.assertEqual(self.notebook.resolve_file(rel_path, page), file_obj)
+		img = tree.find_element('img')
+		rel_path = img.attrib.get('src')
+		self.assertEqual(self.notebook.resolve_file(rel_path, page), file)
 
 	def testCopyPageLinkPasteAsParseTree(self):
 		page = self.notebook.get_page(Path('Test:wiki'))
@@ -302,13 +296,13 @@ some <b>bold</b> text
 			'<zim-tree><p>'
 			'<link href="+Foo">+Foo</link>\n'
 			'<link href="+Foo">my foo link</link>\n'
-			'<link href="+Foo#anchor">+Foo#anchor</link>\n'
+			'<link href="+Foo#anchor">+Foo#anchor</link>'
 			'</p></zim-tree>'
 		) # No need to update the link
 
 	def testCopyPasteParseTreeWithLinkInDifferentPage(self):
 		page = self.notebook.get_page(Path('Test'))
-		page.parse('wiki', '[[+Foo]]\n[[+Foo|Foo]]\n[[+Foo|my foo link]]\n[[+Foo#anchor]]')
+		page.parse('wiki', '[[+Foo]]\n[[+Foo|Foo]]\n[[+Foo|my foo link]]\n[[+Foo#anchor]]\n[[#anchor]]')
 		parsetree = page.get_parsetree()
 		Clipboard.set_parsetree(self.notebook, page, parsetree)
 
@@ -320,12 +314,13 @@ some <b>bold</b> text
 			'<link href="Test:Foo">Foo</link>\n'
 			'<link href="Test:Foo">my foo link</link>\n'
 			'<link href="Test:Foo#anchor">Test:Foo#anchor</link>\n'
+			'<link href="Test#anchor">Test#anchor</link>'
 			'</p></zim-tree>'
 		) # Link updated to point to same target from new location
 
 	def testCopyPasteParseTreeWithLinkInDifferentNotebook(self):
 		page = self.notebook.get_page(Path('Test'))
-		page.parse('wiki', '[[+Foo]]\n[[+Foo|my foo link]]\n[[+Foo#anchor]]')
+		page.parse('wiki', '[[+Foo]]\n[[+Foo|my foo link]]\n[[+Foo#anchor]]\n[[#anchor]]')
 		parsetree = page.get_parsetree()
 		Clipboard.set_parsetree(self.notebook, page, parsetree)
 
@@ -337,6 +332,7 @@ some <b>bold</b> text
 			'<link href="first_notebook?Test:Foo">first_notebook?Test:Foo</link>\n'
 			'<link href="first_notebook?Test:Foo">my foo link</link>\n'
 			'<link href="first_notebook?Test:Foo#anchor">first_notebook?Test:Foo#anchor</link>\n'
+			'<link href="first_notebook?Test#anchor">first_notebook?Test#anchor</link>'
 			'</p></zim-tree>'
 		) # Link updated to point to same target from new location
 
@@ -352,7 +348,7 @@ some <b>bold</b> text
 			'<zim-tree><p>'
 			'<link href="./attachment.pdf">./attachment.pdf</link>\n'
 			'<link href="./attachment.pdf">my attachment</link>\n'
-			'<link href="file://host/file">file://host/file</link>\n'
+			'<link href="file://host/file">file://host/file</link>'
 			'</p></zim-tree>'
 		) # No need to update the link - file uri to external host untouched
 
@@ -368,11 +364,11 @@ some <b>bold</b> text
 			'<zim-tree><p>'
 			'<link href="%s">%s</link>\n'
 			'<link href="%s">my attachment</link>\n'
-			'<link href="file://host/file">file://host/file</link>\n'
+			'<link href="file://host/file">file://host/file</link>'
 			'</p></zim-tree>' % (
-				convert_path_sep('../Test/attachment.pdf'),
-				convert_path_sep('../Test/attachment.pdf'),
-				convert_path_sep('../Test/attachment.pdf')
+				os_native_path('../Test/attachment.pdf'),
+				os_native_path('../Test/attachment.pdf'),
+				os_native_path('../Test/attachment.pdf')
 			)
 		) # Link updated to point to same target from new location - file uri to external host untouched
 
@@ -392,13 +388,13 @@ some <b>bold</b> text
 			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n'
 			'<zim-tree><p>'
 			'<link href="%(file_uri)s">%(file_uri)s</link>\n'
-			'<link href="%(file_uri)s">my attachment</link>\n'
+			'<link href="%(file_uri)s">my attachment</link>'
 			'</p></zim-tree>' % {'file_uri': file_uri}
 		) # Link updated to point to same target from new location
 
 	def testCopyPasteParseTreeWithImageInSamePage(self):
 		page = self.notebook.get_page(Path('Test'))
-		page.parse('wiki', '{{./attachment.png}}\n{{../OtherPage/otherimage.png}}')
+		page.parse('wiki', '{{./attachment.png}}\n{{../OtherPage/otherimage.png}}\n{{./broken_image.png}}')
 		parsetree = page.get_parsetree()
 		Clipboard.set_parsetree(self.notebook, page, parsetree)
 
@@ -408,6 +404,7 @@ some <b>bold</b> text
 			'<zim-tree><p>'
 			'<img src="./attachment.png" />\n'
 			'<img src="../OtherPage/otherimage.png" />\n'
+			'<img src="./broken_image.png" />'
 			'</p></zim-tree>'
 		) # No need to update the images
 
@@ -415,7 +412,7 @@ some <b>bold</b> text
 		self.notebook = self.setUpNotebook(name='first notebook', content=('Test',), mock=tests.MOCK_ALWAYS_REAL)
 
 		page = self.notebook.get_page(Path('Test'))
-		page.parse('wiki', '{{./attachment1.png}}\n{{attachment2.png}}\n{{../OtherPage/otherimage.png}}\n{{./../OtherPage/otherimage.png}}')
+		page.parse('wiki', '{{./attachment1.png}}\n{{attachment2.png}}\n{{../OtherPage/otherimage.png}}\n{{./../OtherPage/otherimage.png}}\n{{./broken_image.png}}')
 		page.attachments_folder.file('attachment1.png').touch()
 		page.attachments_folder.file('attachment2.png').touch()
 
@@ -424,10 +421,13 @@ some <b>bold</b> text
 			newfile = newpage.attachments_folder.file(name)
 			self.assertFalse(newfile.exists())
 
+		self.assertFalse(page.attachments_folder.file('broken_image.png').exists())
+
 		parsetree = page.get_parsetree()
 		Clipboard.set_parsetree(self.notebook, page, parsetree)
 
-		newtree = Clipboard.get_parsetree(self.notebook, Path('OtherPage'))
+		with tests.LoggingFilter('zim.notebook.updater', 'File not found:'):
+			newtree = Clipboard.get_parsetree(self.notebook, Path('OtherPage'))
 		self.assertEqual(newtree.tostring(),
 			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n'
 			'<zim-tree><p>'
@@ -435,11 +435,13 @@ some <b>bold</b> text
 			'<img src="%s" />\n'
 			'<img src="%s" />\n'
 			'<img src="%s" />\n'
+			'<img src="%s" />'
 			'</p></zim-tree>' % (
-				convert_path_sep('./attachment1.png'),
-				convert_path_sep('./attachment2.png'),
-				convert_path_sep('./otherimage.png'),
-				convert_path_sep('./otherimage.png'),
+				os_native_path('./attachment1.png'),
+				os_native_path('./attachment2.png'),
+				os_native_path('./otherimage.png'),
+				os_native_path('./otherimage.png'),
+				os_native_path('./broken_image.png'),
 			)
 		)
 		# No update on two attachments, *but* file is copied
@@ -471,10 +473,10 @@ some <b>bold</b> text
 			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n'
 			'<zim-tree><p>'
 			'<img src="%s" />\n'
-			'<img src="%s" />\n'
+			'<img src="%s" />'
 			'</p></zim-tree>' % (
-				convert_path_sep('./attachment.png'),
-				convert_path_sep('./otherimage.png')
+				os_native_path('./attachment.png'),
+				os_native_path('./otherimage.png')
 			)
 		) # For cross-notebook copy, copy update both images
 		self.assertTrue(newfile_1.exists())
@@ -490,7 +492,7 @@ some <b>bold</b> text
 		self.assertEqual(newtree.tostring(),
 			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n'
 			'<zim-tree><p>'
-			'<object src="./foo.png" type="image+equation" />\n'
+			'<object src="./foo.png" type="image+equation" />'
 			'</p></zim-tree>'
 		) # No need to update the images
 
@@ -517,8 +519,8 @@ some <b>bold</b> text
 		self.assertEqual(newtree.tostring(),
 			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n'
 			'<zim-tree><p>'
-			'<object src="%s" type="image+equation" />\n'
-			'</p></zim-tree>' % convert_path_sep('./foo002.png')
+			'<object src="%s" type="image+equation" />'
+			'</p></zim-tree>' % os_native_path('./foo002.png')
 		) # Sources are copied and number is added due to existing files
 		self.assertTrue(newfile_1.exists())
 		self.assertTrue(newfile_2.exists())
@@ -545,8 +547,8 @@ some <b>bold</b> text
 		self.assertEqual(newtree.tostring(),
 			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n'
 			'<zim-tree><p>'
-			'<object src="%s" type="image+equation" />\n'
-			'</p></zim-tree>' % convert_path_sep('./foo.png')
+			'<object src="%s" type="image+equation" />'
+			'</p></zim-tree>' % os_native_path('./foo.png')
 		) # Sources are copied and number is added due to existing files
 		self.assertTrue(newfile_1.exists())
 		self.assertTrue(newfile_2.exists())
@@ -563,7 +565,7 @@ some <b>bold</b> text
 			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n'
 			'<zim-tree><p>'
 			'<link href="wp?Foo">wp?Foo</link>\n'
-			'<link href="wp?Foo">wikipedia Foo</link>\n'
+			'<link href="wp?Foo">wikipedia Foo</link>'
 			'</p></zim-tree>'
 		) # Does not need update - check it is left alone
 
@@ -581,7 +583,7 @@ some <b>bold</b> text
 			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n'
 			'<zim-tree><p>'
 			'<link href="wp?Foo">wp?Foo</link>\n'
-			'<link href="wp?Foo">wikipedia Foo</link>\n'
+			'<link href="wp?Foo">wikipedia Foo</link>'
 			'</p></zim-tree>'
 		) # Does not need update - check it is left alone
 
@@ -592,9 +594,12 @@ class TestURIData(tests.TestCase):
 		someobject = tests.MockObject()
 		someobject.uri = "file:///foo"
 
+		file1 = LocalFile(os_native_path("/foo"))
+		file2 = LocalFile("~/foo")
+		assert file2.userpath.startswith('~')
 		for (input, uris, text) in (
-			((File("/foo"),), (File("/foo").uri,), File("/foo").path),
-			((File("~/foo"),), (File("~/foo").uri,), "~/foo"),
+			((file1,), (file1.uri,), file1.path),
+			((file2,), (file2.uri,), file2.userpath),
 			(("file:///foo",), ("file:///foo",), "file:///foo"),
 			((someobject,), ("file:///foo",), "file:///foo"),
 			(("file:///foo", "file:///bar"), ("file:///foo", "file:///bar"), "file:///foo file:///bar"),
