@@ -87,6 +87,18 @@ class TestLineSorterWindowExtension(tests.TestCase, TextBufferTestCaseMixin):
 		self.extension.move_line_up()
 		self.assertEqual(self.get_text(), 'B line\nA line\nC line\n')
 
+	def testMoveUpStyledTextIntoCheckboxList(self):
+		self.set_text('[ ] Checkbox #1\n[ ] Checkbox #2\nLine with **bold** text\n')
+		self.place_cursor(32)
+		self.extension.move_line_up()
+		self.assertBufferEqual(self.buffer, '[ ] Checkbox #1\nLine with <strong>bold</strong> text\n[ ] Checkbox #2\n')
+
+	def testMoveUpLineWithPageLinkIntoList(self):
+		self.set_text('* List item #1\n* List item #2\nLine containing a [[page link]]\n')
+		self.place_cursor(30)
+		self.extension.move_line_up()
+		self.assertBufferEqual(self.buffer, '* List item #1\nLine containing a <link href="None">page link</link>\n* List item #2\n')
+
 	def testMoveDownNoSelection(self):
 		self.set_text('A line\nB line\nC line\n')
 		self.place_cursor(10)
