@@ -1,5 +1,5 @@
 
-# Copyright 2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2012,2024 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 
 
@@ -16,6 +16,12 @@ from zim.gui.clipboard import Clipboard, SelectionClipboard
 class TestQuickNotePlugin(tests.TestCase):
 
 	def assertRun(self, args, text):
+		# Bootstrap via "handle_local_commandline" to emulate
+		# how application will handle this
+		cmd = QuickNotePluginCommand('quicknote')
+		cmd.parse_options(*args)
+		args = cmd.handle_local_commandline(list(args))
+
 		cmd = QuickNotePluginCommand('quicknote')
 		cmd.parse_options(*args)
 		dialog = cmd.run()
@@ -24,12 +30,11 @@ class TestQuickNotePlugin(tests.TestCase):
 		buffer = dialog.textview.get_buffer()
 		start, end = buffer.get_bounds()
 		result = start.get_text(end)
-		self.assertTrue(text in result)
+		self.assertIn(text, result)
 
 	def testMain(self):
 		# Text on commandline
 		text = 'foo bar baz\ndus 123'
-		self.assertRun(('text=' + text,), text)
 		self.assertRun(('--text', text), text)
 
 		encoded = 'Zm9vIGJhciBiYXoKZHVzIDEyMwo='
@@ -43,19 +48,13 @@ class TestQuickNotePlugin(tests.TestCase):
 		SelectionClipboard.clipboard.clear() # just to be sure
 		SelectionClipboard.clipboard.set_text('', -1) # HACK to clear it
 		Clipboard.set_text(text)
-		self.assertRun(('input=clipboard',), text)
 		self.assertRun(('--input', 'clipboard',), text)
 
 		text = 'foo bar baz\ndus 456'
 		SelectionClipboard.set_text(text)
-		self.assertRun(('input=clipboard',), text)
 		self.assertRun(('--input', 'clipboard',), text)
 
 		# Template options
-		cmd = QuickNotePluginCommand('quicknote')
-		cmd.parse_options('option:url=foo')
-		self.assertEqual(cmd.template_options, {'url': 'foo'})
-
 		cmd = QuickNotePluginCommand('quicknote')
 		cmd.parse_options('--option', 'url=foo')
 		self.assertEqual(cmd.template_options, {'url': 'foo'})
