@@ -4,6 +4,8 @@
 import tests
 from tests import os_native_path
 
+import os
+
 from zim.fs import adapt_from_oldfs
 from zim.newfs import LocalFile, LocalFolder
 from zim.notebook import Path
@@ -347,6 +349,7 @@ class TestFileDialog(tests.TestCase):
 	## and still it fails at random :(
 	## Maybe fixes in Gtk3 - let's see if we encounter more failures
 
+	@tests.expectedFailureIf(os.name == 'nt') # Fails at random in automated tests
 	def runTest(self):
 		tmp_dir = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
 
