@@ -10,16 +10,18 @@ import contextlib
 
 import logging
 
+from zim.parse.encode import url_encode
+
 logger = logging.getLogger('zim.newfs')
 
 
 from . import FS_SUPPORT_NON_LOCAL_FILE_SHARES
 
 from zim.errors import Error
-from zim.parsing import url_encode, url_decode
+from zim.parse.encode import url_decode
 
 
-is_url_re = re.compile('^\w{2,}:/')
+is_url_re = re.compile(r'^\w{2,}:/')
 is_share_re = re.compile(r'^\\\\\w')
 
 
@@ -107,7 +109,7 @@ def _split_file_url(url):
 	if path.startswith('/localhost/'): # exact 2 '/' before 'localhost'
 		path = path[11:]
 		isshare = False
-	elif scheme == 'smb' or re.match('^/\w', path): # exact 2 '/' followed by hostname
+	elif scheme == 'smb' or re.match(r'^/\w', path): # exact 2 '/' followed by hostname
 		isshare = True
 	else:
 		isshare = False # either 'file:/' or 'file:///'
@@ -580,6 +582,13 @@ except ImportError:
 	logger.debug("Can not import 'xdg.Mime' - falling back to 'mimetypes'")
 	import mimetypes
 
+#: Extensions to determine text mimetypes - used in L{File.istext()}
+TEXT_EXTENSIONS = {
+	'txt': 'text/plain',
+	'md': 'text/markdown',
+	'markdown': 'text/markdown',
+}
+
 #: Extensions to determine image mimetypes - used in L{File.isimage()}
 IMAGE_EXTENSIONS = {
 	# Gleaned from Gdk.get_formats()
@@ -623,8 +632,11 @@ IMAGE_EXTENSIONS = {
 def get_mimetype_from_path(path):
 	if '.' in path:
 		_, ext = path.rsplit('.', 1)
-		if ext.lower() in IMAGE_EXTENSIONS:
-			return IMAGE_EXTENSIONS[ext.lower()]
+		ext = ext.lower()
+		if ext in TEXT_EXTENSIONS:
+			return TEXT_EXTENSIONS[ext]
+		elif ext in IMAGE_EXTENSIONS:
+			return IMAGE_EXTENSIONS[ext]
 
 	if xdgmime:
 		mimetype = xdgmime.get_type(path, name_pri=80)
@@ -664,15 +676,27 @@ class File(FSObjectBase):
 	def __iter__(self):
 		return iter(self.readlines())
 
+	def istext(self):
+		'''Check if this file is a text file
+		Convenience function for checking mimetype starts with 'text/'
+		Works even when no real mime-type suport is available.
+		@returns: C{True} when this is a text file
+		'''
+		if '.' in self.basename:
+			_, ext = self.basename.rsplit('.', 1)
+			if ext.lower() in TEXT_EXTENSIONS:
+				return True
+
+		return self.mimetype().startswith('text/')
+
 	def isimage(self):
-		'''Check if this is an image file. Convenience method that
-		works even when no real mime-type suport is available.
+		'''Check if this file is an image file
+		Convenience function for checking mimetype starts with 'image/'
+		Works even when no real mime-type suport is available.
 		If this method returns C{True} it is no guarantee
 		this image type is actually supported by Gtk.
 		@returns: C{True} when this is an image file
 		'''
-		# Quick shortcut to be able to load images in the gui even if
-		# we have no proper mimetype support
 		if '.' in self.basename:
 			_, ext = self.basename.rsplit('.', 1)
 			if ext.lower() in IMAGE_EXTENSIONS:
